C              PROGRAMME INTPSPEC.FOR
C
C INTPSPEC.FOR finds the spectral density estimate for the VLBI pole path,
C interpolated onto 3,513 equally spaced time points by natural spline
C interpolation, for a record segment of 26.25 years in length.
C The fast Fourier transform is found by calls to the subroutines FFTN
C and FFT2N and then converted to the DFT, after the sequence is reduced
C to zero mean, and Parzen windowed to prevent finite record effects by
C a call to the subroutine WINDOW. The spectral density is given by
C the normalized squared magnitude of the DFT. The output file intpspec.dat
C gives the spectral density estimate as a function of frequency.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DOUBLE PRECISION FREQ(4000),SD(4000),DAY(4000),OMEGA,ARG,AI,
     1 PI,T,T0,AN,DT,ANP,W
      DIMENSION GT(4000),WK1(20000),WK2(20000),GF(4000)
      OPEN(UNIT=1,FILE='ppintp.dat',STATUS='OLD')
      OPEN(UNIT=2,FILE='intpspec.dat',STATUS='UNKNOWN')
C Set values of pi and 2pi.
      PI=3.141592653589793D0
      OMEGA=2.D0*PI
C Set segment length.
      T=26.25D0
C Set midpoint.
      T0=17.125D0
C Set number of interpolates, N.
      N=3513
      AN=DFLOAT(N)
C Find time domain sample interval.
      DT=T/AN
C Find number of positive harmonics.
      NP=(N-1)/2
      ANP=DFLOAT(NP)
C Set mean of pole path to zero.
      GTM=(0.D0,0.D0)
C Read in interpolated pole path.
      DO 10 I=1,N
        READ(1,11)J,IYYY,MM,ID,DAY(I),GT(I)
 11     FORMAT(1X,4I5,F12.7,2F12.6)
C Accumulate sum of co-ordinates.
        GTM=GTM+GT(I)
 10   CONTINUE
C Find mean of pole positions.
      GTM=GTM/AN
C Reduce co-ordinates to zero mean and window sequence.
      DO 12 I=1,N
        GT(I)=GT(I)-GTM
        CALL WINDOW(DAY(I),T0,T,W)
        GT(I)=GT(I)*W
 12   CONTINUE
C Prepare to calculate spectral density.
      IS=-1
C Calculate FFT.
      CALL FFTN(N,GT,IS,L,WK1,WK2)
C Convert to DFT.
      DO 13 I=1,N
C Find index.
        IN=I-NP-1
        AI=DFLOAT(IN)
C Calculate frequency.
        FREQ(I)=AI/T
C Test for negative index, replace with positive index one period ahead.
        IF(IN.LT.0)IN=IN+N
        J=IN+1
C Apply phase shift to find DFT from FFT.
        ARG=OMEGA/AN
        ARG=ARG*AI*NP
        GF(I)=GT(J)*DCMPLX(DCOS(ARG),DSIN(ARG))
C Normalize DFT.
        GF(I)=DT*GF(I)
C Calculate spectral density.
        SD(I)=GF(I)*DCONJG(GF(I)) 
        SD(I)=SD(I)*560.D0/(151.D0*T)
C Write out spectral density.
        WRITE(2,14)FREQ(I),SD(I)
 14     FORMAT(1X,D15.5,D26.15)
 13   CONTINUE
      END
C
      SUBROUTINE WINDOW(T,T0,AM,W)
C
C WINDOW finds the value of the PARZEN window, W, at time T,
C for a window of length AM centred at time T0.
C 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Find relative time on (-1,1) time base.
      X=2.D0*(T-T0)/AM
C Find unity minus absolute value of relative time.
      OMX=1.D0-DABS(X)
C If absolute value of relative time is greater than 0.5,
C select alternate cubic.
      IF(DABS(X).GT.0.5D0) GO TO 10
      W=1.D0-6.D0*OMX*X*X
      GO TO 20
 10   W=2.D0*OMX*OMX*OMX
 20   CONTINUE   
      RETURN
      END
C
      SUBROUTINE FFTN(LG,G,IS,L,WK1,WK2)
C FFTN finds the discrete Fourier transform of the sequence G of length LG
C for input IS=-1. For input IS=1, it finds the inverse of the DFT
C sequence G of length LG. LG can be any positive integer. The subroutine
C uses the method of Bluestein in which the transform is converted to
C a convolution. The sequences entering the convolution are padded with
C zeroes to length L, an integral power of two, equal to or larger than 3LG-2.
C The convolution is then performed by finding the DFTs of each sequence
C using the subroutine FFT2N, multiplying them, and then finding the inverse
C of the resulting DFT. WK1 and WK2 are work area vectors, defined in
C the main programme, of length L or larger. On output, the transformed
C sequence replaces the input sequence.
C  
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DOUBLE PRECISION PI,ARG,ARGJ2,FACT,DC,DS
      DIMENSION G(LG),WK1(L),WK2(L)
C Define pi.
      PI=3.14159265358979D0
C Define common factor in argument of W.
      ARG=PI*DFLOAT(IS)/DFLOAT(LG)
C Determine if DFT or inverse DFT.
      FACT=1.D0
      IF(IS.EQ.1) FACT=FACT/DFLOAT(LG)
C Find N, the power of two equal to, or next larger than 3*LG-2,
C the length of the convolution.
      LGT3M2=3*LG-2
      N=0
      I=1
 10   IF(I.GE.LGT3M2) GO TO 20
      I=I*2
      N=N+1
      GO TO 10
 20   CONTINUE
      L=2**N
C Zero fill padded sequences being convolved.
      DO 30 I=1,L
        WK1(I)=(0.D0,0.D0)
        WK2(I)=(0.D0,0.D0)
 30   CONTINUE
C Find sequences to be convolved.
      DO 40 J=1,LG
        JM1=J-1
        JM1S=JM1*JM1
        ARGJ2=ARG*DFLOAT(JM1S)
        DC=DCOS(ARGJ2)
        DS=DSIN(ARGJ2)
        WK1(J)=FACT*G(J)*DCMPLX(DC,DS)
C Store weights for final DFT calculation.
        G(J)=DCMPLX(DC,DS)
        JPLGM1=J+LG-1
        WK2(JPLGM1)=DCMPLX(DC,-DS)
        IF(J.EQ.1) GO TO 40
        LGMJP1=LG-J+1
        WK2(LGMJP1)=WK2(JPLGM1)
 40   CONTINUE
C Find FFTs of sequences entering convolution.
      IS=-1
      CALL FFT2N(L,WK1,IS)
      CALL FFT2N(L,WK2,IS)
C Multiply FFTs to find FFT of convolution.
      DO 50 I=1,L
        WK1(I)=WK1(I)*WK2(I)
 50   CONTINUE
C Find convolution by inverse DFT.
      IS=1
      CALL FFT2N(L,WK1,IS)
C Weight convolution to obtain DFT.
      DO 60 I=1,LG
        IPLGM1=I+LG-1
        G(I)=G(I)*WK1(IPLGM1)
 60   CONTINUE
      RETURN   
      END
C
      SUBROUTINE FFT2N(LG,G,IS)
C FFT2N finds the discrete Fourier transform of the sequence G of length
C equal an integral power, N, of 2, for input IS=-1. For input IS=1,
C it finds the inverse of the DFT sequence G of length LG, again an integral
C power, N, of 2. The input sequences are first decimated into basic pairs,
C which are then successively transformed recursively by doubling to
C the final resulting sequence. On output, the resulting sequence replaces
C the input sequence.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DOUBLE PRECISION PI,ARG,ARGM,FACT
      DIMENSION G(LG)
C Define pi.
      PI=3.14159265358979D0
C Define common factor in argument of W.
      ARG=PI*DFLOAT(IS)
C Determine if DFT or inverse DFT.
      FACT=1.D0
      IF(IS.EQ.1) FACT=FACT/DFLOAT(LG)
C Begin sequence decimation and scaling.
      J=1
      DO 10 I=1,LG
        IF(I.GT.J) GO TO 20
        HOLD=G(J)*FACT
        G(J)=G(I)*FACT
        G(I)=HOLD
 20     M=LG/2
 30     IF(J.LE.M) GO TO 40
        J=J-M
        M=M/2
        IF(M.GE.1) GO TO 30
 40     J=J+M
 10   CONTINUE
C If input sequence is of unit length, exit.
      IF(LG.EQ.1) GO TO 70
C Start calculation of DFTs by doubling.
C Set initial DFT length as power of 2.
      L=1
 50   LT2=L*2
      DO 60 M=1,L
        MM1=M-1
C Find argument of W raised to required power.
        ARGM=DFLOAT(MM1)*ARG/DFLOAT(L)
C Find W to required power.
        W=DCMPLX(DCOS(ARGM),DSIN(ARGM))
C Find double length transforms.
        DO 60 K=M,LG,LT2
          HOLD=W*G(K+L)
          G(K+L)=G(K)-HOLD
          G(K)=G(K)+HOLD
 60   CONTINUE
C Update DFT length.
      L=LT2
C Test if transform is complete.
      IF(L.LT.LG) GO TO 50
 70   CONTINUE
      RETURN   
      END

